@@ -3,25 +3,26 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.1 (the "License").  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON- INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+#ifndef RLD
 #include "stdio.h"
+#endif /* !defined(RLD) */
 #include "stdlib.h"
 #include "string.h"
 #include <mach/mach.h>
@@ -41,6 +42,16 @@ static const struct arch_flag arch_flags[] = {
     { "any",	CPU_TYPE_ANY,	  CPU_SUBTYPE_MULTIPLE },
     { "little",	CPU_TYPE_ANY,	  CPU_SUBTYPE_LITTLE_ENDIAN },
     { "big",	CPU_TYPE_ANY,	  CPU_SUBTYPE_BIG_ENDIAN },
+
+/* 64-bit Mach-O architectures */
+
+    /* architecture families */
+    { "ppc64",     CPU_TYPE_POWERPC64, CPU_SUBTYPE_POWERPC_ALL },
+    /* specific architecture implementations */
+    { "ppc970-64", CPU_TYPE_POWERPC64, CPU_SUBTYPE_POWERPC_970 },
+
+/* 32-bit Mach-O architectures */
+
     /* architecture families */
     { "ppc",    CPU_TYPE_POWERPC, CPU_SUBTYPE_POWERPC_ALL },
     { "i386",   CPU_TYPE_I386,    CPU_SUBTYPE_I386_ALL },
@@ -60,11 +71,13 @@ static const struct arch_flag arch_flags[] = {
     { "ppc750", CPU_TYPE_POWERPC, CPU_SUBTYPE_POWERPC_750 },
     { "ppc7400",CPU_TYPE_POWERPC, CPU_SUBTYPE_POWERPC_7400 },
     { "ppc7450",CPU_TYPE_POWERPC, CPU_SUBTYPE_POWERPC_7450 },
+    { "ppc970", CPU_TYPE_POWERPC, CPU_SUBTYPE_POWERPC_970 },
     { "i486",   CPU_TYPE_I386,    CPU_SUBTYPE_486 },
     { "i486SX", CPU_TYPE_I386,    CPU_SUBTYPE_486SX },
     { "pentium",CPU_TYPE_I386,    CPU_SUBTYPE_PENT }, /* same as i586 */
     { "i586",   CPU_TYPE_I386,    CPU_SUBTYPE_586 },
-    { "pentpro", CPU_TYPE_I386, CPU_SUBTYPE_PENTPRO },
+    { "pentpro", CPU_TYPE_I386, CPU_SUBTYPE_PENTPRO }, /* same as i686 */
+    { "i686",   CPU_TYPE_I386, CPU_SUBTYPE_PENTPRO },
     { "pentIIm3",CPU_TYPE_I386, CPU_SUBTYPE_PENTII_M3 },
     { "pentIIm5",CPU_TYPE_I386, CPU_SUBTYPE_PENTII_M5 },
     { "m68030", CPU_TYPE_MC680x0, CPU_SUBTYPE_MC68030_ONLY },
@@ -75,6 +88,7 @@ static const struct arch_flag arch_flags[] = {
     { NULL,	0,		  0 }
 };
 
+#ifndef RLD
 /*
  * get_arch_from_flag() is passed a name of an architecture flag and returns
  * zero if that flag is not known and non-zero if the flag is known.
@@ -112,6 +126,7 @@ void)
 {
 	return(arch_flags);
 }
+#endif /* !defined(RLD) */
 
 /*
  * get_arch_name_from_types() returns the name of the architecture for the
@@ -133,9 +148,14 @@ cpu_subtype_t cpusubtype)
 	       arch_flags[i].cpusubtype == cpusubtype)
 		return(arch_flags[i].name);
 	}
+#ifndef RLD
 	p = savestr("cputype 1234567890 cpusubtype 1234567890");
 	if(p != NULL)
 	    sprintf(p, "cputype %u cpusubtype %u", cputype, cpusubtype);
+#else
+	/* there is no sprintf() in the rld kernel API's */
+	p = savestr("cputype ?? cpusubtype ??");
+#endif
 	return(p);
 }
 
@@ -172,6 +192,7 @@ const struct arch_flag *flag)
    if(flag->cputype == CPU_TYPE_MC680x0 ||
       flag->cputype == CPU_TYPE_MC88000 ||
       flag->cputype == CPU_TYPE_POWERPC ||
+      flag->cputype == CPU_TYPE_POWERPC64 ||
       flag->cputype == CPU_TYPE_HPPA ||
       flag->cputype == CPU_TYPE_SPARC ||
       flag->cputype == CPU_TYPE_I860 ||
@@ -183,6 +204,7 @@ const struct arch_flag *flag)
         return UNKNOWN_BYTE_SEX;
 }
 
+#ifndef RLD
 /*
  * get_stack_direction_from_flag() returns the direction the stack grows as
  * either positive (+1) or negative (-1) of the architecture for the
@@ -259,6 +281,7 @@ const struct arch_flag *flag)
 
     return(64*1024*1024);
 }
+#endif /* !defined(RLD) */
 
 /*
  * get_segalign_from_flag() returns the default segment alignment (page size).
@@ -269,7 +292,8 @@ get_segalign_from_flag(
 const struct arch_flag *flag)
 {
 	if(flag->cputype == CPU_TYPE_POWERPC ||
-	   flag->cputype == CPU_TYPE_VEO)
+	   flag->cputype == CPU_TYPE_VEO ||
+	   flag->cputype == CPU_TYPE_I386)
 	    return(0x1000); /* 4K */
 	else
 	    return(0x2000); /* 8K */
