@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -84,8 +82,8 @@ struct indr_object {
 #define INITIAL_LIST_SIZE	250
 struct list {
     struct indr_object **list;	/* the order list */
-    long used;			/* the number used in the list */
-    long size;			/* the current size of the list */
+    unsigned long used;		/* the number used in the list */
+    unsigned long size;		/* the current size of the list */
 };
 static struct list indr_list;
 
@@ -192,7 +190,8 @@ int argc,
 char *argv[],
 char *envp[])
 {
-    unsigned long i, j;
+    int i;
+    unsigned long j;
     enum bool no_flags_left;
     char *list_filename, *output_file, *input_file;
     struct arch_flag *arch_flags;
@@ -605,7 +604,8 @@ struct object *object)
     long new_nsyms;
     struct nlist *new_symbols;
 
-	if(object->mh->filetype == MH_DYLIB){
+	if(object->mh->filetype == MH_DYLIB ||
+	   object->mh->filetype == MH_DYLIB_STUB){
 	    if(member != NULL)
 		fatal_arch(arch, member, "is a dynamic library which is not "
 			   "allowed as a member of an archive");
@@ -646,7 +646,7 @@ struct object *object)
 	    if(nlistp->n_type & N_EXT){
 		if(nlistp->n_un.n_strx){
 		    if(nlistp->n_un.n_strx > 0 &&
-		       nlistp->n_un.n_strx < strings_size){
+		       (unsigned long)nlistp->n_un.n_strx < strings_size){
 			sp = lookup_symbol(strings + nlistp->n_un.n_strx);
 			if(sp != NULL){
 			    if(sp->type == N_UNDF)
@@ -668,7 +668,7 @@ struct object *object)
 	    else{
 		if(nlistp->n_un.n_strx){
 		    if(nlistp->n_un.n_strx > 0 && nlistp->n_un.n_strx <
-								strings_size)
+							    (long)strings_size)
 			nlistp->n_un.n_strx = add_to_string_table(
 				strings + nlistp->n_un.n_strx);
 		    else
@@ -889,7 +889,7 @@ struct object *object)
 	for(i = 0; i < nsyms; i++){
 	    if(symbols[i].n_un.n_strx != 0){
 		if(symbols[i].n_un.n_strx < 0 ||
-		   symbols[i].n_un.n_strx > strsize){
+		   (unsigned long)symbols[i].n_un.n_strx > strsize){
 		    error_arch(arch, NULL, "bad string index for symbol "
 			       "table entry %ld in: ", i);
 		    return;
@@ -1414,7 +1414,7 @@ void
 make_indr_objects(
 struct arch *arch)
 {
-    long i;
+    unsigned long i;
     cpu_type_t cputype;
     cpu_subtype_t cpusubtype;
     enum byte_sex target_byte_sex;
